package examples.extractor;

import java.util.Map;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
public class ExtractorUtils {

    public static Double division(String s1, String s2, Map<String, String> maps){
        double rate = 0.0;
        if (maps.containsKey(s1) && maps.containsKey(s2)){
            Double d1 = Double.valueOf(maps.get(s1));
            if (isDoubleEqualToZero(d1)){
                return rate;
            }
            Double d2 = Double.valueOf(maps.get(s2));
            rate = d2 / d1;
        }
        return rate;
    }
    public static Double divisionDouble(Double d1, Double d2){
        double rate = 0.0;
        if (isDoubleEqualToZero(d1)){
            return rate;
        }
        rate = d2 / d1;
        return rate;
    }

    public static boolean isDoubleEqualToZero(double value) {
        final double epsilon = 1e-10; // 定义一个很小的误差范围
        // 判断value是否在误差范围内
        return Math.abs(value) < epsilon;
    }

    public static double ceilLog(Double key) {
        double bucket = Math.ceil(Math.log(key + 1.0) * 100);
        if (bucket > 100L) {
            bucket = 100L;
        }
        if (bucket < 0) {
            bucket = 0;
        }
        return  (double)bucket;
    }

    public static double bucketRatioFeature(Double key) {
//        long bucket = Math.round(Math.log((key + 1.0) * 10));
//        if (bucket > 50L) {
//            bucket = 50L;
//        }
        double bucket = Math.round(Math.pow(key, 0.5) * 100);
        if (bucket > 100L) {
            bucket = 100L;
        }
        if (bucket < 0) {
            bucket = 0;
        }
        return (double)bucket;
    }

    public static double calculateVariance(List<Double> numbers) {
        double average = numbers.stream()
                .mapToDouble(Double::doubleValue)
                .average()
                .orElse(0.0);

        double squaredDiffSum = numbers.stream()
                .mapToDouble(Double::doubleValue)
                .map(x -> Math.pow(x - average, 2))
                .average()
                .orElse(0.0);

        return squaredDiffSum;
    }

    public static double calculateAverage(List<Double> numbers) {
        if (numbers == null || numbers.isEmpty()) {
            return 0.0;
        }
        return numbers.stream()
                .mapToDouble(Number::doubleValue)
                .average()
                .orElse(0.0);
    }

    public static List<Double> calculateDifferences(List<Double> numbers) {
        List<Double> differences = new ArrayList<>();

        for (int i = 0; i < numbers.size() - 1; i++) {
            Double diff = numbers.get(i + 1) - numbers.get(i);
            differences.add(diff);
        }

        return differences;
    }

    public static List<String> generateHourStrings(String timeString, int N) {
        LocalDateTime dateTime = LocalDateTime.parse(timeString, DateTimeFormatter.ofPattern("yyyyMMddHH"));
        List<String> hourStrings = new ArrayList<>();
        for (int i = 0; i < N; i++) {
            hourStrings.add(dateTime.minusHours(i).format(DateTimeFormatter.ofPattern("yyyyMMddHH")));
        }

        return hourStrings;
    }

    public static String subtractHours(String inputDateTime, int hoursToSubtract) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMddHH");
        LocalDateTime dateTime = LocalDateTime.parse(inputDateTime, formatter);
        LocalDateTime subtractedDateTime = dateTime.minusHours(hoursToSubtract);
        return subtractedDateTime.format(formatter);
    }

    public static void main(String[] args) {
        System.out.println(bucketRatioFeature(0.1));
        System.out.println(bucketRatioFeature(0.8));
        System.out.println(bucketRatioFeature(0.01));
        System.out.println(bucketRatioFeature(0.007));

    }

}
